/***********************************************************************************************************************
 *
 * blueBill Stats
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile/
 * SCM: http://java.net/hg/bluebill-server~stats-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.stats.ui.spi;

import javax.annotation.Nonnull;
import it.tidalwave.util.Finder;
import it.tidalwave.role.SimpleComposite;
import it.tidalwave.role.spi.DefaultSimpleComposite;
import it.tidalwave.role.ui.PresentationModelFactory;
import it.tidalwave.bluebill.stats.domain.Ping;
import it.tidalwave.bluebill.stats.domain.PingDao;
import it.tidalwave.bluebill.stats.ui.PingsView;
import it.tidalwave.bluebill.stats.ui.PingsViewController;
import org.springframework.core.task.TaskExecutor;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import static it.tidalwave.util.Finder.SortDirection.*;
import static it.tidalwave.util.ui.UserNotification.*;
import static it.tidalwave.bluebill.stats.domain.PingFinder.*;

/***********************************************************************************************************************
 *
 * The default implementation of {@link PingsViewController}s.
 * 
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor @Slf4j
public class DefaultPingsViewController implements PingsViewController
  {
    @Nonnull
    private final PingDao dao;
            
    @Nonnull
    private final PingsView view;

    @Nonnull
    private final TaskExecutor executor;
    
    @Nonnull
    private final PresentationModelFactory presentationModelFactory;
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override
    public void initialize() 
      {
        log.info("initialize()");
        view.lock(notification().withText("Loading data...")); // TODO: use a bundle
        
        executor.execute(new Runnable() 
          {
            @Override
            public void run()
              {
                final Finder<Ping> finder = dao.findPings().sort(BY_TIMESTAMP, DESCENDING);
                final SimpleComposite<Ping> composite = new DefaultSimpleComposite<Ping>(finder);
                view.populate(presentationModelFactory.createPresentationModel(new Object[]{ composite }));
                view.unlock();
              }
          });
      }
  }
